/*************************************************************************
   AntiRight
   (c) 2003 Jeffrey Bedard
   jefbed@e-list.net
 
   This file is part of AntiRight.

    AntiRight is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    AntiRight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AntiRight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* This is AntiRight. AntiRight is a program that is meant to provide
   a lightweight control panel for the system.  AntiRight makes it
   easier to perform everyday system commands from the push of a
   button.  AntiRight was developed with Lesstif.
   
   This program is distributed under the terms of the GNU General
   Public Lecense.  To find out more, look at the file called COPYING
   in the project root directory.  */



/* This is the version control header.  */
/* 	$Id: anticlock.c,v 1.11 2003/10/10 22:42:30 jefbed Exp $  */

#include <time.h>
#include "../antiright-src/antiright.h"
#include "anticlock.xbm"

Widget anticlock_label;
Window window;
Display *display;
XGCValues gc_values;
GC gc;

/* These animations are creaping featurism and they will be removed
   immediately.  */
void
draw_line(int x1, int y1,
	  int x2, int y2)
{
  XDrawLine(display, window, gc, x1, y1, x2, y2);
}

void anticlock_draw_star(int x, int y)
{
  draw_line(x, y, x+8, y);
  draw_line(x+8, y, x, y+8);
  draw_line(x, y+8, x+4, y-4);
  draw_line(x+4, y-4, x+8, y+8);
  draw_line(x+8, y+8, x, y);
}

void
anticlock_draw_day(int x, int y)
{
  XFillArc(display, window, gc, x, y, 24, 24, 0, 360*64);

  draw_line(x, y+12, x-10, y+12);
  draw_line(x+24, y+12, x+34, y+12);
  draw_line(x+12, y, x+12, y-10);
  draw_line(x+12, y+24, x+12, y+34);
  
  draw_line(x-10, y+12, x+12, y);
  draw_line(x+34, y+12, x+12, y+24);
  draw_line(x+12, y-10, x+24, y+12);
  draw_line(x+12, y+34, x, y+12);
}

void
anticlock_draw_night(int x, int y)
{
  XDrawArc(display, window, gc, x, y, 32, 32, 90*64, 180*64);
  XDrawArc(display, window, gc, x+12, y-16, 64, 64, 152*64, 56*64);
  
  anticlock_draw_star(x+36, y);
  anticlock_draw_star(x+24, y+16);
  anticlock_draw_star(x+36, y+32);
  
  anticlock_draw_star(x-12, y);
}


void 
anticlock_draw_hand(int x, int thickness)
{
  XSetLineAttributes(display, gc, thickness, LineSolid, CapRound, JoinRound);
  draw_line(128, 32, x, 100);
}

void
antiright_update_clock()
{
  char *time_string;
  time_t time_date;
  struct tm time_struct;
  int x;

  (void)time(&time_date);
  time_string=ctime(&time_date);

  time_string[strlen(time_string)-1]=' ';
  
  time_struct=*localtime(&time_date);

  XClearWindow(display,
	       window);
  XSetFont(display, gc,
	   XLoadFont(display,
		     "-*-helvetica-bold-r-normal-*-18-*-*-*-*-*-*-*"));
  XDrawString(display,
	      window,
	      gc,
	      16,20,
	      time_string, strlen(time_string));
  XFillArc(display, window, gc,
	   120, 24,
	   16, 16,
	   0, 360*64);
  XSetLineAttributes(display, gc,
		     6, LineSolid, CapRound, JoinRound);
  /* Hour hand.  */
  x=abs(((256/24)*(time_struct.tm_hour+1)));
  anticlock_draw_hand(x, 6);
  /* Minute hand.  */
  x=abs((((256/60)*(time_struct.tm_min+1))));
  anticlock_draw_hand(x, 4);
  /* Second hand.  */
  x=abs(((256/60)*(time_struct.tm_sec+1)));
  anticlock_draw_hand(x, 2);
  
  
  
  if(time_struct.tm_hour>17)
    {
      anticlock_draw_night(200, 40);  
    }
  else
    {
      anticlock_draw_day(200, 40);
    }
    
  XFlush(display);
}

void
antiright_clock_update_work_procedure(XtPointer client_data,
				      XtIntervalId* timer_id)
{
  antiright_update_clock();
  *timer_id=
    XtAppAddTimeOut(app_context,
		    1000, /* 1 second.  */
		    antiright_clock_update_work_procedure,
		    client_data);
}

int
main(int argc, char* argv[])    
{
  Widget menu_widget;
  Widget menubar_widget;
  Widget mainwindow_widget;
  
  antiright_initialize_application(argc, argv);
  
  antiright_set_icon(antiright_parent_widget,
		     "AntiClock",
		     anticlock_bits,
		     anticlock_width,
		     anticlock_height);

  mainwindow_widget= 
    XtVaCreateManagedWidget("Row", 
			    xmRowColumnWidgetClass, 
			    antiright_parent_widget, 
			    XmNorientation, 
			    XmVERTICAL,
			    NULL);

  antiright_number_arguments=0;
  menubar_widget= 
    XmCreateMenuBar(mainwindow_widget, 
		    "menubar", 
		    antiright_arguments, 
		    antiright_number_arguments);
  XtManageChild(menubar_widget);
  menu_widget = antiright_create_pulldown_menu(menubar_widget, "File");
  antiright_create_button(menu_widget, 
			  "Close", 
			  exit_cb, 
			  "Close");
  
  menu_widget=antiright_create_help_menu(menubar_widget);
  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNwidth, 
	   256);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNheight, 
	   108);
  antiright_number_arguments++;
  anticlock_label=XmCreateDrawingArea(mainwindow_widget,
				      "AntiClock",
				      antiright_arguments,
				      antiright_number_arguments);
  XtAddCallback(anticlock_label,
		XmNexposeCallback,
		antiright_update_clock,
		NULL);
  XtAddCallback(anticlock_label,
		XmNinputCallback,
		antiright_update_clock,
		NULL);
  XtAddCallback(anticlock_label,
		XmNresizeCallback,
		antiright_update_clock,
		NULL);
  XtManageChild(anticlock_label);  
  XtManageChild(menubar_widget);
  XtManageChild(mainwindow_widget);

  XtRealizeWidget(antiright_parent_widget);
 

  display=XtDisplay(anticlock_label);
  window=XtWindow(anticlock_label);
  
      
  gc=XCreateGC(display,
	       window,
	       0,
	       NULL);
  
  XFlush(display); /* I don't know why this works here.  */
 
  XtAppAddTimeOut(app_context,
		  1000,
		  antiright_clock_update_work_procedure,
		  NULL);
  antiright_update_clock();
    
  XtAppMainLoop(app_context);
  return(0);
}















