/*************************************************************************
   AntiRight
   (c) 2003 Jeffrey Bedard
   jefbed@e-list.net
 
   This file is part of AntiRight.

    AntiRight is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    AntiRight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AntiRight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* This file contains the library functions used throughout Antiright */

/*Actually, what I'd like is a little toy spaceship!!*/

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>    
#include <Xm/Label.h>
#include <Xm/PushB.h> 
#include <Xm/TextF.h>
#include <Xm/Text.h>     
#include <Xm/RowColumn.h>   
#include <Xm/ScrollBar.h>
#include <Xm/RowColumn.h>
#include <Xm/MessageB.h>
#include <Xm/Separator.h>
#include <stdlib.h>
#include <stdio.h>
#include "help.h"
#include "antiright.h"
#include "library.h"




/* Functions start here.  */

void 
ok_cb(Widget widget,
      XtPointer client_data,
      XtPointer call_data)
{
  char *filename;
  filename = client_data;
  XtUnmanageChild(widget);
  call_data=(XtPointer)NULL;
}
void
antiright_check_return(int return_value)
{
  if(return_value != 0)
    {
      if(antiright_status_widget != NULL)
	{
	  XmTextInsert(antiright_status_widget,
		       XmTextGetInsertionPosition(antiright_status_widget),
		       "\nFunction returned bad value.");
	}
      DEBUG("Function returned bad value");
    }
}

/*  This is my strcat, used for portability and because of the fact
    that most strcat implementations do not work as I want them to
    work.  */
/*  Use the returned string!  Make sure to free it.  */
char *antiright_strcat(char *head, char *tail)
{
  unsigned int counter;
  char *string=(char*)malloc(strlen(head)+strlen(tail));
  if(string != NULL)
    {
      strcpy(string, head);
      for(counter=0; counter<(strlen(tail)+1); counter++)
	{
	  string[strlen(head)+counter]=tail[counter];
	}
      return(string);
    }
  else
    {
      DEBUG("Unable to allocate memory");
      return(NULL);
    }
}

void 
cancel_cb(Widget widget,
	  XtPointer client_data,
	  XtPointer call_data)
{
  client_data = (XtPointer)NULL;
  call_data= (XtPointer)NULL;
  XtUnmanageChild(widget);
  XtDestroyWidget(widget);
}

/* In theses callback declarations, cb stands for callback, obviously.  */
void 
antiright_command_cb(Widget widget,
		     Widget* client_data,
		     XtPointer call_data)
{
  XtPointer command_string;

  command_string=XmTextFieldGetString(*client_data);
  puts(command_string);
  antiright_system_cb(NULL, command_string, NULL);
  putchar('\n');
  XtFree(command_string);
  widget=(Widget)NULL;
  call_data=(XtPointer)NULL;
}

void 
exit_cb()
{
  exit(0);
}

/* This opens a new window by starting a new instance of the
   program.  */
void 
antiright_new_cb()
{
  /* Short and sweet.  */
  system("antiright &");
}


void 
antiright_system_cb(Widget widget, 
		    XtPointer client_data, 
		    XtPointer call_data)
{
  widget = (Widget)NULL;
  call_data = (XtPointer)NULL;
  /* This code is commented out as it is the source of a segmentation
     fault.  */
  /*if(antiright_status_widget != NULL)
    {
      XmTextInsert(antiright_status_widget,
		   XmTextGetInsertionPosition(antiright_status_widget),
		   "\n");
      XmTextInsert(antiright_status_widget,
		   XmTextGetInsertionPosition(antiright_status_widget),
		   client_data);
		   }*/
  antiright_check_return(system(client_data));
}

/* This is a function that greatly shortens the time needed to create
   a menu item on a pulldown menu.  */
Widget 
antiright_create_pulldown_menu(Widget parent_widget, 
			       char* name)
{
  Widget menu;
  Widget cascade;

  menu= 
    XmCreatePulldownMenu(parent_widget, 
			 name, 
			 antiright_arguments, 
			 antiright_number_arguments);
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNsubMenuId, 
	   menu); 
  antiright_number_arguments++;
  cascade= 
    XmCreateCascadeButton(parent_widget, 
			  name, 
			  antiright_arguments, 
			  antiright_number_arguments);
  XtManageChild(cascade);
  return(menu);
}



/* This shortens the time needed to create a pushbutton by performing
   all of the drudgery beforehand.  */
/* Use only one button creation function.  */
Widget
antiright_create_button(Widget parent_widget, 
			char* name, 
			XtCallbackProc callback, 
			XtPointer client_data)
{
  Widget button_widget;
  button_widget= 
    XmCreatePushButton(parent_widget, 
		       name, 
		       antiright_arguments, 
		       antiright_number_arguments);
  XtAddCallback(button_widget, 
		XmNactivateCallback, 
		callback, 
		client_data);
  XtManageChild(button_widget);
  return(button_widget);
}

void 
antiright_create_label(Widget parent_widget, 
		       char* name)
{
    Widget label;
    label=
	XmCreateLabel(parent_widget,
		      name,
		      antiright_arguments,
		      antiright_number_arguments);
    XtManageChild(label);
}



void 
antiright_create_separator(Widget parent)
{
    Widget separator;
    separator=
	XmCreateSeparator(parent,
			  "separator",
			  antiright_arguments,
			  antiright_number_arguments);
    XtManageChild(separator);
}
void
antiright_set_icon(Widget parent_widget,
		   const char *name,
		   const char* bits,
		   int width,
		   int height)
{
  Display *display;
  Pixmap icon;

  display=XtDisplay(parent_widget);

  icon=
    XCreateBitmapFromData(display, 
			  RootWindow(display, DefaultScreen(display)), 
			  (const char*)bits, 
			  width, 
			  height);

  XtVaSetValues(parent_widget, 
		XmNiconName, 
		name, 
		XmNiconPixmap, 
		icon, 
		NULL);
}

/* Handle arguments and set up parent_widget.  */
void
antiright_initialize_application(int argc, 
				 char **argv)
{
   /* This is simplistic command line argument handling. */
  if(argc>1) /* This prevents a segmentation fault.  */
    {
      
      if(!strcmp(argv[1],"-h") 
	 | !strcmp(argv[1],"-help") 
	 | !strcmp(argv[1],"--help"))
	{
	  puts("AntiRight Desktop Environment");
	  puts("Copyright (C) 2003 Jeffrey Bedard");
	  puts("AntiRight is a lightweight dekstop environment.  ");
	  puts("It is distributed under the GNU General Public License.");
	  puts("There is NO WARRANTY.  Please see COPYING for details.");
	  puts("Command Line Options:");
	  /* Note that some options are not included as either they do not
	     work, I dont want them, or they would screw up
	     operations.  */
	  puts("\t-display");
	  puts("\t-geometry");
	  puts("\t-bg, -background");
	  puts("\t-fg, -foreground");
	  puts("\t-iconic");
	  puts("\t-synchronous");
	  puts("\t-title");
	  puts("\t-xrm");
	  puts("Report bugs to <jefbed@e-list.net>");
	  exit(0);
	}
    }
  
  antiright_parent_widget=
    XtVaOpenApplication(&app_context, 
			"AntiRight", /* Default Class is AntiRight.  */
			NULL, 
			0, 
			&argc, 
			argv, 
			NULL,
			sessionShellWidgetClass,
			NULL,
			NULL);
}

Widget
antiright_create_help_menu(Widget menubar_widget)
{
  Widget menu_widget;
  Widget cascade_widget;
  antiright_number_arguments=0;
  menu_widget=
    XmCreatePulldownMenu(menubar_widget, 
			 "Help", 
			 antiright_arguments, 
			 antiright_number_arguments);
  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNsubMenuId, 
	   menu_widget); 
  antiright_number_arguments++;
  cascade_widget=
    XmCreateCascadeButton(menubar_widget,
			  "Help", 
			  antiright_arguments, 
			  antiright_number_arguments);
  XtManageChild(cascade_widget);
  XtVaSetValues(menubar_widget, 
		XmNmenuHelpWidget, 
		cascade_widget, 
		NULL);
  antiright_create_button
    (menu_widget, 
     "Tutorial", 
     antiright_system_cb, 
     "xmmessage \"`cat /usr/local/share/doc/antiright/TUTORIAL`\" &");
  antiright_create_button(menu_widget, 
			  "On Version", 
			  antiright_help_cb, 
			  VERSION);
  antiright_create_button(menu_widget, 
			  "No Warranty", 
			  antiright_help_cb, 
			  "There is NO warranty; USE AT YOUR OWN RISK!");
  antiright_create_button(menu_widget, 
			  "Copyright", 
			  antiright_help_cb, 
			  "Copyright 2003, Jeffrey E. Bedard");
  antiright_create_button
    (menu_widget, 
     "License", 
     antiright_system_cb, 
     "xmmessage \"`cat /usr/local/share/doc/antiright/COPYING`\" &");
  antiright_create_button(menu_widget, 
			  "About AntiRight", 
			  antiright_help_cb,
			  "AntiRight is a lightweight desktop environment.");
  return(menu_widget);
}

			   


