
/*************************************************************************
   AntiRight
   (c) 2003 Jeffrey Bedard
   jefbed@e-list.net
 
   This file is part of AntiRight.

    AntiRight is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    AntiRight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AntiRight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/*
	arcd.c   - AntiRightCD
*/

#include "../antiright-src/antiright.h"
#include <fcntl.h>
#include <sys/ioctl.h>
#ifdef HAVE_LINUX
#include <linux/cdrom.h>
#endif /* HAVE_LINUX */
#include "ARCD.xbm"

/* This program's psuedo-namespace is "arcd".  */

/* status_label + parent is global to allow other functions to change
   its text.  */
Widget arcd_parent_widget;
Widget arcd_status_label;
/* CD-ROM file descriptor id.  */
int cdrom_file_descriptor;
/* Track list structure.  */
#ifdef HAVE_LINUX
struct cdrom_tochdr cdrom_track_list;
#endif /* HAVE_LINUX */
/* Track index structure.  */
#ifdef HAVE_LINUX
struct cdrom_ti cdrom_tracks;
#endif /* HAVE_LINUX */
/* CD-ROM is ejected flag.  */
Boolean cdrom_is_ejected;
/* CD-ROM is paused flag. */
Boolean cdrom_is_paused;
/* Current track.  */
int current_track;


void
initialize_cdrom()
{
  if(cdrom_file_descriptor != 0)
    {
#ifdef HAVE_LINUX
      close(cdrom_file_descriptor);
#endif /* HAVE_LINUX */
    }
#ifdef HAVE_LINUX
  cdrom_file_descriptor=open("/dev/cdrom", O_RDONLY | O_NONBLOCK);
  ioctl(cdrom_file_descriptor, CDROMREADTOCHDR, &cdrom_track_list);
  current_track=cdrom_tracks.cdti_trk0+1;
  cdrom_tracks.cdti_ind0=0;
  cdrom_tracks.cdti_ind1=255;
  cdrom_tracks.cdti_trk1=cdrom_track_list.cdth_trk1;
#endif /* HAVE_LINUX */
}

void
play_cb()
{
#ifdef HAVE_LINUX
  cdrom_tracks.cdti_trk0=current_track;
#endif /* HAVE_LINUX */
  if(!cdrom_is_paused)
    {
#ifdef HAVE_LINUX
      ioctl(cdrom_file_descriptor, CDROMPLAYTRKIND, &cdrom_tracks);
#endif /* HAVE_LINUX */
    }
  else
    {
#ifdef HAVE_LINUX
      ioctl(cdrom_file_descriptor, CDROMRESUME);
#endif /* HAVE_LINUX */
      cdrom_is_paused=False;
    }
}

void
stop_cb()
{
#ifdef HAVE_LINUX
  ioctl(cdrom_file_descriptor, CDROMSTOP);
#endif /* HAVE_LINUX */
  initialize_cdrom();
}

void
pause_cb()
{
  if(cdrom_is_paused==False)
    {
      cdrom_is_paused=True;
#ifdef HAVE_LINUX
      ioctl(cdrom_file_descriptor, CDROMPAUSE);
#endif /* HAVE_LINUX */
    }
  else
    {
      cdrom_is_paused=False;
#ifdef HAVE_LINUX
      ioctl(cdrom_file_descriptor, CDROMRESUME);
#endif /* HAVE_LINUX */
    }
}


void
eject_cb()
{
  if(cdrom_is_ejected==True)
    {
      /* Close CD.  */
#ifdef HAVE_LINUX
      ioctl(cdrom_file_descriptor, CDROMCLOSETRAY);
#endif /* HAVE_LINUX */
      initialize_cdrom();
      cdrom_is_ejected=False;
    }
  else
    {
      /* Eject CD.  */
#ifdef HAVE_LINUX
      ioctl(cdrom_file_descriptor, CDROMEJECT);
#endif /* HAVE_LINUX */
      cdrom_is_ejected=True;
    }
  initialize_cdrom();
}

void
update_status()
{
  int digit_number=0, dec, sign;
  XmString label_string;

#ifdef HAVE_LINUX
  label_string=
    XmStringConcat(XmStringCreateLocalized(fcvt(cdrom_tracks.cdti_trk0, 
						digit_number,&dec, &sign)),
		   XmStringConcat
		   (XmStringCreateLocalized("/"),
		    XmStringCreateLocalized(fcvt
					    (cdrom_track_list.cdth_trk1, 
					     digit_number,&dec, &sign))));
#else
  label_string=
    XmStringCreateLocalized("AntiRight CD Player");
#endif /* HAVE_LINUX */

  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNlabelString, 
	   label_string); 
  antiright_number_arguments++;
  XtSetValues(arcd_status_label,
	      antiright_arguments,
	      antiright_number_arguments);
  antiright_number_arguments=0;
  XmStringFree(label_string);
}

void
next_cb()
{
#ifdef HAVE_LINUX
  current_track=cdrom_tracks.cdti_trk0;
  ioctl(cdrom_file_descriptor, CDROMSTOP);
#endif /* HAVE_LINUX */
  current_track++;
#ifdef HAVE_LINUX
  cdrom_tracks.cdti_ind0=0;
#endif /* HAVE_LINUX */
  play_cb();
  update_status();
}

void
last_cb()
{
#ifdef HAVE_LINUX
  current_track=cdrom_tracks.cdti_trk0;
  ioctl(cdrom_file_descriptor, CDROMSTOP);
#endif /* HAVE_LINUX */
  current_track--;
#ifdef HAVE_LINUX
  cdrom_tracks.cdti_ind0=0;
#endif /* HAVE_LINUX */
  play_cb();
  update_status();
}



void
update_status_work_procedure(XtPointer client_data,
			     XtIntervalId* timer_id)
{
  update_status();
  *timer_id=
    XtAppAddTimeOut(app_context,
		    1000, /* 1 second.  */
		    update_status_work_procedure,
		    client_data);
}

int
main(int argc, char* argv[])    
{
  Widget mainwindow_widget;
  Widget toolbar;
  Widget exit_widget;
  Widget eject_widget;
  Widget play_widget; 
  Widget stop_widget; 
  Widget pause_widget;
  Widget next_widget;
  Widget last_widget;
  XmString label_string;
  
  
  antiright_initialize_application(argc, argv);
  

  antiright_set_icon(antiright_parent_widget,
		     "ARCD",
		     (const char*)ARCD_bits, 
		     ARCD_width,
		     ARCD_height);
    
  mainwindow_widget = 
    XtVaCreateManagedWidget("Row", xmFormWidgetClass, 
			    antiright_parent_widget, 
			    XmNorientation, XmVERTICAL, 
			    NULL);
  
  
  label_string=XmStringCreateLocalized("AntiRight CD");
  
  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNleftAttachment, XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNrightAttachment, XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNbottomAttachment, XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNlabelString,
	   label_string);
  antiright_number_arguments++;
  arcd_status_label=
    XmCreateLabel(mainwindow_widget, 
		  "arcd_status_label",
		  antiright_arguments,
		  antiright_number_arguments);
  XmStringFree(label_string);
  XtManageChild(arcd_status_label);

  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNbottomAttachment, XmATTACH_WIDGET);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNbottomWidget,  arcd_status_label);
  antiright_number_arguments++;
  
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNleftAttachment, XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNrightAttachment, XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNtopAttachment, XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNorientation, XmHORIZONTAL);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNpacking, XmPACK_COLUMN);
  antiright_number_arguments++;
  toolbar= 
    XmCreateRowColumn(mainwindow_widget, 
		      "toolbar", 
		      antiright_arguments, antiright_number_arguments);
  XtManageChild(toolbar);
  
  
  
  antiright_number_arguments=0;
  last_widget=XmCreatePushButton(toolbar, "<<", 
				 antiright_arguments, antiright_number_arguments);
  XtAddCallback(last_widget, XmNactivateCallback, last_cb, NULL);
  XtManageChild(last_widget);
  
  antiright_number_arguments=0;
  play_widget=XmCreatePushButton(toolbar, ">", 
				 antiright_arguments, antiright_number_arguments);
  XtAddCallback(play_widget, XmNactivateCallback, play_cb, NULL);
  XtManageChild(play_widget);
  antiright_number_arguments=0;
  next_widget=XmCreatePushButton(toolbar, ">>", 
				 antiright_arguments, antiright_number_arguments);
  XtAddCallback(next_widget, XmNactivateCallback, next_cb, NULL);
  XtManageChild(next_widget);
  
  antiright_number_arguments=0;
  pause_widget=XmCreatePushButton(toolbar, "||", 
				  antiright_arguments, antiright_number_arguments);
  XtAddCallback(pause_widget, XmNactivateCallback, pause_cb, NULL);
  XtManageChild(pause_widget);
  
  antiright_number_arguments=0;
  stop_widget=XmCreatePushButton(toolbar, "S", 
				 antiright_arguments, antiright_number_arguments);
  XtAddCallback(stop_widget, XmNactivateCallback, stop_cb, NULL);
  XtManageChild(stop_widget);
  
  antiright_number_arguments=0;
  eject_widget=XmCreatePushButton(toolbar, "^", 
				  antiright_arguments, antiright_number_arguments);
  XtAddCallback(eject_widget, XmNactivateCallback, eject_cb, NULL);
  XtManageChild(eject_widget);
  
  antiright_number_arguments=0;
  exit_widget=
    XmCreatePushButton(toolbar, 
		       "X", 
		       antiright_arguments, antiright_number_arguments);
  XtAddCallback(exit_widget,
		XmNactivateCallback, 
		exit_cb,
		"Exit");
  XtManageChild(exit_widget);
  
  
  XtManageChild(mainwindow_widget);
  XtRealizeWidget(antiright_parent_widget);

  initialize_cdrom();
  XtAppAddTimeOut(app_context,
		  1000, /* 1 second.  */
		  update_status_work_procedure,
		  NULL);
  XtAppMainLoop(app_context);
  return(0);
}

