
/*************************************************************************
   AntiRight
   (c) 2003 Jeffrey Bedard
   jefbed@e-list.net
 
   This file is part of AntiRight.

    AntiRight is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    AntiRight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AntiRight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "xmtextedit.xbm"
#include "../antiright-src/antiright.h"

Widget antiright_scrolled_text;

typedef struct
{
  Boolean file_is_opened;
  char *filename;
  XmString file_text;
}file_environment_struct;
file_environment_struct file_environment;

Boolean antiright_textedit_clear_flag;

void 
save_cb()
{
  if(file_environment.file_is_opened==True)
    {
      FILE* fp;
      char* text;
      fp=fopen(file_environment.filename, "w");
      if(fp!=NULL)
	{
	  text=XmTextGetString(antiright_scrolled_text);
	  fprintf(fp, "%s\n", text);
	  XtFree(text);
	  fclose(fp);
	}
      else
	{
	  DEBUG("Failed to open file");
	}
    }
}
void 
file_open(char *filename_argument)
{
  XmString label_string;
  struct stat file_info;
  FILE *file_pointer;
  char* buffer;
  long bytes_read;

  file_pointer = fopen(filename_argument, "r"); /* Open the file to
						   read into the text
						   area. */
  if(file_pointer != NULL)
    {
      stat(filename_argument, &file_info); /* Get file information.  */
      buffer = (char*)XtMalloc(file_info.st_size);
      bytes_read=
	fread(buffer, 
	      1, 
	      file_info.st_size, 
	      file_pointer);
  
      fclose(file_pointer);
      XmTextSetString(antiright_scrolled_text, buffer);
      XtFree(buffer);
    }

  file_environment.filename=filename_argument;
  file_environment.file_is_opened=True;
  label_string=
    XmStringCreateLocalized(filename_argument);
  XtVaSetValues(antiright_status_widget,
		XmNlabelString,
		label_string,
		NULL);
  XmStringFree(label_string);
}

void 
open_ok_cb(Widget widget, 
	   XtPointer client_data, 
	   XtPointer call_data)
{
  XmFileSelectionBoxCallbackStruct *callback_struct_pointer;

  
  callback_struct_pointer=
    call_data;
  client_data=NULL;
  if(file_environment.file_is_opened==True)
    {
      save_cb();
    };		
  XmStringGetLtoR(callback_struct_pointer->value, 
		  XmSTRING_DEFAULT_CHARSET,  
		  &file_environment.filename);
  file_open(file_environment.filename);
  XtUnmanageChild(widget);
  XtDestroyWidget(widget);  
}
void
open_cancel_cb(Widget widget, 
	       XtPointer client_data, 
	       XtPointer call_data)
{
  client_data=NULL;
  call_data=NULL;
  
  XtUnmanageChild(widget);
  XtDestroyWidget(widget);
}

void 
open_cb()
{
  Widget fsb;
  
  fsb=
    XmCreateFileSelectionDialog(antiright_parent_widget,
				"FileSB",
				antiright_arguments,
				antiright_number_arguments);
  
  XtAddCallback(fsb, 
		XmNokCallback, 
		open_ok_cb, 
		"OK");	
  XtAddCallback(fsb,
		XmNhelpCallback,
		antiright_help_cb,
		"This dialog is used to open a file for editing");
  XtAddCallback(fsb, 
		XmNcancelCallback, 
		open_cancel_cb, 
		"Cancel");					
  
  XtManageChild(fsb);				
}

void 
search_ok_cb(Widget widget, 
	     XtPointer client_data, 
	     XtPointer call_data)
{
  XmTextPosition position;
  XmSelectionBoxCallbackStruct *prompt_data;
  char *search_string;

  client_data=NULL;
  prompt_data=call_data;
  XmStringGetLtoR(prompt_data->value,
		  XmSTRING_DEFAULT_CHARSET,
		  &search_string);
  XmTextFindString(antiright_scrolled_text,
		   XmTextGetInsertionPosition(antiright_scrolled_text),
		   search_string,
		   XmTEXT_FORWARD,
		   &position);
  XmTextSetInsertionPosition(antiright_scrolled_text, 
			     position);
  XtFree(search_string);
  widget=NULL;
  call_data=NULL;
}


void 
search_cb()
{
  Widget prompt;

  antiright_number_arguments=0;
  prompt=
    XmCreatePromptDialog(antiright_parent_widget,
			 "Search",
			 antiright_arguments,
			 antiright_number_arguments);
  XtAddCallback(prompt,
		XmNokCallback,
		search_ok_cb,
		"Search");
  XtAddCallback(prompt,
		XmNhelpCallback,
		antiright_help_cb,
		"Enter the text that you want to find in the text field.");
  XtManageChild(prompt);
}

void 
print_cb()
{
  system(antiright_strcat("lpr ", file_environment.filename));
}

void 
cut_cb()
{
  XmTextCut(antiright_scrolled_text, 
	    XtLastTimestampProcessed(XtDisplay(antiright_scrolled_text)));
}
void 
copy_cb()
{
  XmTextCopy(antiright_scrolled_text, 
	     XtLastTimestampProcessed(XtDisplay(antiright_scrolled_text)));
}
void 
paste_cb()
{
  XmTextPaste(antiright_scrolled_text);
}


int
main(int argc, 
     char* argv[])    
{
  Widget menu_widget;
  Widget menubar_widget; 
  Widget mainwindow_widget;
  XmString label_string;
  
  label_string=NULL;
  antiright_initialize_application(argc, argv);

  antiright_set_icon(antiright_parent_widget,
		     "XmTextEdit",
		     xmtextedit_bits,
		     xmtextedit_width,
		     xmtextedit_height);
  
  mainwindow_widget= 
    XtVaCreateManagedWidget("Row", 
			    xmFormWidgetClass, 
			    antiright_parent_widget, 
			    XmNorientation, 
			    XmVERTICAL, 
			    NULL);
  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNleftAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNrightAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNtopAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  menubar_widget= 
    XmCreateMenuBar(mainwindow_widget, 
		    "menubar", 
		    antiright_arguments, 
		    antiright_number_arguments);
  menu_widget= 
    antiright_create_pulldown_menu(menubar_widget, 
				   "File");
  antiright_create_button(menu_widget,
			  "New",
			  antiright_system_cb,
			  "xmtextedit &");
  antiright_create_button(menu_widget, 
			  "Open", 
			  open_cb, 
			  "Open");
  antiright_create_button(menu_widget, 
			  "Save", 
			  save_cb, 
			  "Save");
  antiright_create_button(menu_widget,
			  "Print",
			  print_cb,
			  "Print");				
  antiright_create_button(menu_widget, 
			  "Close", 
			  exit_cb, 
			  "Close");
  menu_widget=antiright_create_pulldown_menu(menubar_widget, "Edit");
  antiright_create_button(menu_widget,
			  "Search",
			  search_cb,
			  "Search");
  antiright_create_button(menu_widget,
			  "Cut",
			  cut_cb,
			  "Cut");
  antiright_create_button(menu_widget,
			  "Copy",
			  copy_cb,
			  "Copy");
  antiright_create_button(menu_widget,
			  "Paste",
			  paste_cb,
			  "Paste");
  menu_widget=antiright_create_help_menu(menubar_widget);

  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNleftAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNrightAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNbottomAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;

  antiright_status_widget=
    XmCreateLabel(mainwindow_widget, 
		  "No files are currently open.  ",
		  antiright_arguments,
		  antiright_number_arguments);
  XmStringFree(label_string);
  XtManageChild(antiright_status_widget);
  
  
  
  antiright_number_arguments=0;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNeditMode, 
	   XmMULTI_LINE_EDIT); 
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNrows, 
	   24); 
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNcolumns, 
	   80); 
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNscrollVertical, 
	   True); 
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNscrollHorizontal, 
	   True); 
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNbottomAttachment,
	   XmATTACH_WIDGET);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNbottomWidget,  
	   antiright_status_widget);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNleftAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNrightAttachment, 
	   XmATTACH_FORM);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNtopAttachment, 
	   XmATTACH_WIDGET);
  antiright_number_arguments++;
  XtSetArg(antiright_arguments[antiright_number_arguments], 
	   XmNtopWidget,  
	   menubar_widget);
  antiright_number_arguments++;
  antiright_scrolled_text = 
    XmCreateScrolledText(mainwindow_widget, 
			 "antiright_scrolled_text", 
			 antiright_arguments, 
			 antiright_number_arguments);
  XtManageChild(antiright_scrolled_text);
  
  
  
  
  
  XtManageChild(menubar_widget);
  XtManageChild(mainwindow_widget);
  
  /* Open file if specified at command line */
  /* Xt removes its arguments, so no checks needed.  */
  if(argc>1)
    {
      file_open(argv[1]);
    }
  
  
  XtRealizeWidget(antiright_parent_widget);
  XtAppMainLoop(app_context);
  return(0);
}

